from enum import Enum

from pydantic import BaseModel

RgbStr = str  # Like `rgb(255, 99, 132)`
Function = str | None  # Placeholder for what we'll define in JS
Color = str
Script = str  # This is intended to represent a "well known function" provided at a given namespace by a JS library
number = int | float
# TODO: Add specific well known scripts from what we've developed in DIRD?

"""
Color examples for reference:
color: 'green'                  // named color
color: '#dc143c'                // HEX color
color: 'rgb(51, 170, 51)'       // RGB color (opaque)
color: 'rgba(51, 170, 51, .5)'  // RGBa color (semi-transparent)
"""


class Font(BaseModel):
    family: str | None = None
    size: int | None = None
    style: str | None = None
    lineHeight: float | None = None
    weight: str | int | None = None


class PointStyle(str, Enum):
    circle = "circle"
    cross = "cross"
    crossRot = "crossRot"
    dash = "dash"
    line = "line"
    rect = "rect"
    rectRounded = "rectRounded"
    rectRot = "rectRot"
    star = "star"
    triangle = "triangle"


class PaddingObject(BaseModel):
    left: int | None = None
    right: int | None = None
    top: int | None = None
    bottom: int | None = None


class PaddingXY(BaseModel):
    x: int | None = None
    y: int | None = None


Padding = int | PaddingObject | PaddingXY


if __name__ == "__main__":
    PaddingObject().model_dump_json()
    PaddingXY().model_dump_json()



import calendar
from random import randint

from pydacharts.color import ChartColor, ChartColorTransparent
from pydacharts.models import (
    ChartType,
    Config,
    Grid,
    Interaction,
    Legend,
    LineData,
    LineDataSet,
    Options,
    Plugins,
    ScaleOptions,
    Scales,
    StepOption,
    Title,
)


def test_line():
    config = Config(
        type=ChartType.line,
        data=LineData(
            labels=[calendar.month_name[i] for i in range(1, 8)],
            datasets=[
                LineDataSet(
                    label="My First Dataset",
                    data=[65, 59, 80, 81, 56, 55, 40],
                    fill=False,
                    borderColor=ChartColor.GREEN,
                )
            ],
        ),
    )

    assert config.type == ChartType.line
    assert config.data.labels == [calendar.month_name[i] for i in range(1, 8)]
    assert len(config.data.datasets) == 1
    assert config.data.datasets[0].label == "My First Dataset"
    assert config.data.datasets[0].data == [65, 59, 80, 81, 56, 55, 40]
    assert config.data.datasets[0].fill is False
    assert config.data.datasets[0].borderColor == ChartColor.GREEN


def test_multi_axis():
    labels = [calendar.month_name[i] for i in range(1, 8)]
    data1 = [randint(-100, 100) for _ in range(1, 8)]
    data2 = [randint(-100, 100) for _ in range(1, 8)]

    data = LineData(
        labels=labels,
        datasets=[
            LineDataSet(
                label="Dataset 1",
                data=data1,
                borderColor=ChartColor.RED,
                backgroundColor=ChartColorTransparent.RED,
                yAxisID="y",
            ),
            LineDataSet(
                label="Dataset 2",
                data=data2,
                borderColor=ChartColor.BLUE,
                backgroundColor=ChartColorTransparent.BLUE,
                yAxisID="y",
            ),
        ],
    )
    options = Options(
        plugins=Plugins(
            legend=Legend(position="top"),
            title=Title(display=True, text="Chart.js Line Chart - Multi Axis"),
        ),
        interaction=Interaction(mode="index", intersect=False),
        stacked=False,
        scales=Scales(
            y=ScaleOptions(type="linear", display="true", position="left"),
            x=ScaleOptions(type="linear", display="true", position="right"),
            grid=Grid(drawOnChartArea=False),
        ),
    )

    config = Config(type="line", data=data, options=options)

    assert config.type == "line"
    assert config.data.labels == labels
    assert len(config.data.datasets) == 2
    assert config.data.datasets[0].data == data1
    assert config.data.datasets[1].data == data2
    assert config.options.plugins.legend.position == "top"
    assert config.options.plugins.title.display is True
    assert config.options.plugins.title.text == "Chart.js Line Chart - Multi Axis"
    assert config.options.interaction.mode == "index"
    assert config.options.interaction.intersect is False
    assert config.options.stacked is False
    assert config.options.scales.y.type == "linear"
    assert config.options.scales.y.display is True
    assert config.options.scales.y.position == "left"
    assert config.options.scales.x.type == "linear"
    assert config.options.scales.x.display is True
    assert config.options.scales.x.position == "right"
    assert config.options.scales.grid.drawOnChartArea is False


def test_stepped_line():
    labels = [calendar.month_name[i] for i in range(1, 8)]
    data_values = [randint(-100, 100) for _ in range(1, 8)]

    data = LineData(
        labels=labels,
        datasets=[
            LineDataSet(
                label="Dataset 1",
                data=data_values,
                borderColor=ChartColor.RED,
                backgroundColor=ChartColorTransparent.RED,
                fill=False,
                stepped=StepOption.middle,
            )
        ],
    )

    options = Options(
        interaction=Interaction(intersect=False, axis="x"),
        plugins=Plugins(title=Title(display=True, text="Step chart")),
    )

    config = Config(type=ChartType.line, data=data, options=options)

    assert config.type == ChartType.line
    assert config.data.labels == labels
    assert len(config.data.datasets) == 1
    assert config.data.datasets[0].data == data_values
    assert config.data.datasets[0].borderColor == ChartColor.RED
    assert config.data.datasets[0].backgroundColor == ChartColorTransparent.RED
    assert config.data.datasets[0].fill is False
    assert config.data.datasets[0].stepped == StepOption.middle
    assert config.options.interaction.intersect is False
    assert config.options.interaction.axis == "x"
    assert config.options.plugins.title.display is True
    assert config.options.plugins.title.text == "Step chart"


def test_styled_line():
    labels = [calendar.month_name[i] for i in range(1, 8)]
    data1 = [randint(-100, 100) for _ in range(1, 8)]
    data2 = [randint(-100, 100) for _ in range(1, 8)]
    data3 = [randint(-100, 100) for _ in range(1, 8)]

    data = LineData(
        labels=labels,
        datasets=[
            LineDataSet(
                label="Unfilled",
                data=data1,
                borderColor=ChartColor.RED,
                backgroundColor=ChartColorTransparent.RED,
                fill=False,
                stepped=StepOption.middle,
            ),
            LineDataSet(
                label="Dashed",
                data=data2,
                borderColor=ChartColor.BLACK,
                backgroundColor=ChartColorTransparent.BLUE,
                fill=False,
                borderDash=[5, 5],
            ),
            LineDataSet(
                label="Filled",
                data=data3,
                borderColor=ChartColor.GREEN,
                backgroundColor=ChartColorTransparent.GREEN,
                fill=True,
                stepped=True,
            ),
        ],
    )

    options = Options(
        interaction=Interaction(intersect=False, mode="index"),
        plugins=Plugins(title=Title(display=True, text="Chart.js Line Chart")),
        scales=Scales(
            y=ScaleOptions(display="true", title=Title(text="Value")),
            x=ScaleOptions(display="true", title=Title(text="Month")),
        ),
    )

    config = Config(type=ChartType.line, data=data, options=options)

    assert config.type == ChartType.line
    assert config.data.labels == labels
    assert len(config.data.datasets) == 3
    assert config.data.datasets[0].data == data1
    assert config.data.datasets[0].borderColor == ChartColor.RED
    assert config.data.datasets[0].backgroundColor == ChartColorTransparent.RED
    assert config.data.datasets[0].fill is False
    assert config.data.datasets[0].stepped == StepOption.middle
    assert config.data.datasets[1].data == data2
    assert config.data.datasets[1].borderColor == ChartColor.BLACK
    assert config.data.datasets[1].backgroundColor == ChartColorTransparent.BLUE
    assert config.data.datasets[1].fill is False
    assert config.data.datasets[1].borderDash == [5, 5]
    assert config.data.datasets[2].data == data3
    assert config.data.datasets[2].borderColor == ChartColor.GREEN
    assert config.data.datasets[2].backgroundColor == ChartColorTransparent.GREEN
    assert config.data.datasets[2].fill is True
    assert config.data.datasets[2].stepped is True
    assert config.options.interaction.intersect is False
    assert config.options.interaction.mode == "index"
    assert config.options.plugins.title.display is True
    assert config.options.plugins.title.text == "Chart.js Line Chart"
    assert config.options.scales.y.display is True
    assert config.options.scales.y.title.text == "Value"
    assert config.options.scales.x.display is True
    assert config.options.scales.x.title.text == "Month"



from pydacharts.chartjs_types import Font, Function, PaddingObject, number
from pydacharts.elements import BarsElements
from pydacharts.models import (
    DataLabelsPlugin,
    Elements,
    Grid,
    Layout,
    Legend,
    Options,
    Plugins,
    ScaleOptions,
    Scales,
    Ticks,
    Title,
    Tooltip,
    TooltipCallbacks,
)


def chart_options() -> Options:
    return Options(
        responsive=True,
        scales=Scales(
            y=ScaleOptions(grid=Grid(display=True), ticks=Ticks(font=Font(size=14))),
            x=ScaleOptions(grid=Grid(display=False)),
        ),
        plugins=Plugins(
            legend=Legend(display=False),
            tooltip=Tooltip(
                position="cursor", enabled=False, external="externalTooltipHandler"
            ),
        ),
    )


class BarChartOptions(Options):
    barPercentage: float | None = 0.6
    aspectRatio: number | Function | None = "aspectRatioFunction"
    elements: Elements = Elements(bars=BarsElements(borderRadius=10))


class HorizontalBarChartOptions(BarChartOptions):
    indexAxis: str | None = "y"
    plugins: Plugins = Plugins(datalabels=DataLabelsPlugin(align="end", anchor="end"))


class HorizontalBarChartStackedOptions(HorizontalBarChartOptions):
    layout: Layout = Layout(padding=PaddingObject(right=40))
    aspectRatio: Function = (
        "(context) => { return Math.max(2, 10 / context.chart.data.labels.length);"
    )
    scales: Scales = Scales(
        y=ScaleOptions(stacked=True, grid=Grid(display=False)),
        x=ScaleOptions(stacked=True, grid=Grid(display=False)),
    )
    plugins: Plugins = Plugins(
        datalabels=DataLabelsPlugin(
            align="end",
            anchor="end",
            padding=PaddingObject(left=12),
            formatter="(value, ctx) => ctx.chart.$totalizer.totals[ctx.dataIndex]",
            display="(ctx) => ctx.datasetIndex === ctx.chart.$totalizer.utmost",
        )
    )


class PyramidChartOptions(HorizontalBarChartStackedOptions):
    scales: Scales = Scales(
        x=ScaleOptions(
            display=True,
            grid=Grid(display=True),
            ticks=Ticks(
                beginAtZero=True,
                callback="(value) => `${Math.abs(value) / 1000}k`;",
                stepSize=2000,
            ),
        ),
    )
    plugins: Plugins = Plugins(
        datalabels=DataLabelsPlugin(display=False),
        tooltip=Tooltip(
            callbacks=TooltipCallbacks(
                label="""
                 (context) => {
                    let label = context.dataset.label || "";
                    let value = context.formattedValue;
                    return `${label ? label + ": " : ""}${value.split("-").join("")}`;
                    """
            )
        ),
    )


class DoughnutChartOptions(Options):
    cutout: str = "80%"
    scales: Scales = Scales(
        y=ScaleOptions(display=False, grid=Grid(display=False)),
        x=ScaleOptions(display=False, grid=Grid(display=False)),
    )
    plugins: Plugins = Plugins(
        title=Title(padding=20, color="#70798C", font=Font(size=20, weight="normal"))
    )



from pydantic import BaseModel

from pydacharts.models import Config


class SankeyDatasetData(BaseModel):
    from_: str
    to: str
    flow: int


class SankeyDataSet(BaseModel):
    data: list[SankeyDatasetData]
    colorMode: str = "gradient"
    labels: dict[str, str] | None
    priority: dict[str, int] | None


class SankeyData(BaseModel):
    datasets: list[SankeyDataSet]


class Sankey(Config):
    type: str = "sankey"
    data: SankeyData  # type: ignore

    def model_dump_json(self, *args, **kwargs):
        """
        Hack to make python -> json keyword conversion
        """
        data = super().model_dump_json(*args, **kwargs)
        data = data.replace('"from_":', '"from":')
        return data



from typing import Literal

from pydantic import BaseModel, Field

from pydacharts.chartjs_types import Color, PointStyle, number


class PointsElements(BaseModel):
    radius: number | None = Field(None, description="Point radius.")
    pointStyle: PointStyle | None = Field(None, description="Point style.")
    rotation: number | None = Field(None, description="Point rotation (in degrees).")
    backgroundColor: Color | None = Field(None, description="Point fill color.")
    borderWidth: number | None = Field(None, description="Point stroke width.")
    borderColor: Color | None = Field(None, description="Point stroke color.")
    hitRadius: number | None = Field(
        None, description="Extra radius added to point radius for hit detection."
    )
    hoverRadius: number | None = Field(None, description="Point radius when hovered.")
    hoverBorderWidth: number | None = Field(
        None, description="Stroke width when hovered."
    )


class LinesElements(BaseModel):
    tension: number | None = Field(
        None, description="Bézier curve tension (0 for no Bézier curves)."
    )
    backgroundColor: Color | None = Field(None, description="Line fill color.")
    borderWidth: number | None = Field(None, description="Line stroke width.")
    borderColor: Color | None = Field(None, description="Line stroke color.")
    borderCapStyle: str | None = Field(None, description="Line cap style. See MDN")
    borderDash: list[number] | None = Field(None, description="Line dash. See MDN")
    borderDashOffset: number | None = Field(
        None, description="Line dash offset. See MDN"
    )
    borderJoinStyle: Literal["round", "bevel", "miter"] | None = Field(
        None, description="Line join style. See MDN"
    )
    capBezierPoints: bool | None = Field(
        None,
        description="true to keep Bézier control inside the chart, false for no restriction.",
    )
    cubicInterpolationMode: str | None = Field(
        None, description="Interpolation mode to apply. See more..."
    )
    fill: bool | str | None = Field(
        None, description="How to fill the area under the line. See area charts."
    )
    stepped: bool | None = Field(
        None,
        description="true to show the line as a stepped line (tension will be ignored).",
    )


class BarsElements(BaseModel):
    """
    Bar Configuration
    Bar elements are used to represent the bars in a bar chart.
    Namespace: options.elements.bar, global bar options: Chart.defaults.elements.bar.
    """

    backgroundColor: Color | None = Field(None, description="Bar fill color.")
    borderWidth: number | None = Field(None, description="Bar stroke width.")
    borderColor: Color | None = Field(None, description="Bar stroke color.")
    borderSkipped: (
        Literal["start", "end", "middle", "bottom", "left", "top", "right", False]
        | None
    ) = Field(
        None,
        description="Skipped (excluded) border: 'start', 'end', 'middle', 'bottom', 'left', 'top', 'right' or false.",
    )
    borderRadius: number | dict | None = Field(
        None, description="The bar border radius (in pixels)."
    )
    inflateAmount: number | Literal["auto"] | None = Field(
        None,
        description="The amount of pixels to inflate the bar rectangle(s) when drawing.",
    )
    pointStyle: PointStyle | None = Field(
        None, description="Style of the point for legend."
    )


class ArcElements(BaseModel):
    """
    Arc Configuration
    Arcs are used in the polar area, doughnut and pie charts.
    Namespace: options.elements.arc, global arc options: Chart.defaults.elements.arc.
    """

    angle: number | None = Field(None, description="Arc angle to cover.")
    backgroundColor: Color | None = Field(None, description="Arc fill color.")
    borderAlign: Literal["center", "inner"] | None = Field(
        None, description="Arc stroke alignment."
    )
    borderColor: Color | None = Field(None, description="Arc stroke color.")
    borderJoinStyle: Literal["round", "bevel", "miter", "bevel", "round"] | None = (
        Field(None, description="Line join style")
    )
    borderWidth: number | None = Field(None, description="Arc stroke width.")


class Elements(BaseModel):
    """
    arc, lines, points, and bars.
    """

    arc: ArcElements | None = None
    lines: LinesElements | None = None
    points: PointsElements | None = None
    bars: BarsElements | None = None


if __name__ == "__main__":
    arc = ArcElements()
    lines = LinesElements()
    points = PointsElements()
    bars = BarsElements()

    print(Elements(arc=arc, lines=lines, points=points, bars=bars).json())



from pydacharts.models import ChartType, Config


class Doughnut(Config):
    type: str = ChartType.doughnut


class Bar(Config):
    type: str = ChartType.bar


class Line(Config):
    type: str = ChartType.line



import pytest

from pydacharts.elements import ArcElements, BarsElements, LinesElements, PointsElements


@pytest.mark.parametrize(
    "element", [ArcElements, LinesElements, PointsElements, BarsElements]
)
def test_instances(element):
    element().model_dump_json()



from pydacharts.plugins import sankey as s


def test_sankey():
    """
    Intended to work together with the "sankey" charts plugin
    """
    sankey = s.Sankey(
        data=s.SankeyData(
            datasets=[
                s.SankeyDataSet(
                    data=[
                        s.SankeyDatasetData(from_="a", to="b", flow=10),
                        s.SankeyDatasetData(from_="a", to="c", flow=5),
                        s.SankeyDatasetData(from_="b", to="c", flow=2),
                        s.SankeyDatasetData(from_="b", to="d", flow=3),
                    ],
                    priority={"a": 1, "b": 2, "c": 3},
                    labels={"a": "Label 1", "b": "Label 2", "c": "Label 3"},
                )
            ]
        )
    )

    # Assertions
    assert len(sankey.data.datasets) == 1
    dataset = sankey.data.datasets[0]

    assert len(dataset.data) == 4
    assert dataset.data[0].from_ == "a"
    assert dataset.data[0].to == "b"
    assert dataset.data[0].flow == 10
    assert dataset.data[1].from_ == "a"
    assert dataset.data[1].to == "c"
    assert dataset.data[1].flow == 5
    assert dataset.data[2].from_ == "b"
    assert dataset.data[2].to == "c"
    assert dataset.data[2].flow == 2
    assert dataset.data[3].from_ == "b"
    assert dataset.data[3].to == "d"
    assert dataset.data[3].flow == 3

    assert dataset.priority == {"a": 1, "b": 2, "c": 3}
    assert dataset.labels == {"a": "Label 1", "b": "Label 2", "c": "Label 3"}



from flask import Flask, render_template

from .serve_data import config

app = Flask(__name__)


@app.route("/")
def hello_world():
    config_json = config().model_dump_json(exclude_none=True, indent=2)
    return render_template(
        "chart_datalabels.html",
        config=config_json,
    )



from pydacharts.models import ChartType, Config, PieData, PieDataSet


def test_pie():
    config = Config(
        type=ChartType.pie,
        data=PieData(
            labels=["Red", "Orange", "Yellow", "Green", "Blue"],
            datasets=[
                PieDataSet(
                    label="My First Dataset",
                    data=[65, 59, 80, 81, 56],
                    fill=False,
                    backgroundColor=["Red", "Orange", "Yellow", "Green", "Blue"],
                )
            ],
        ),
    )

    assert config.type == ChartType.pie
    assert config.data.labels == ["Red", "Orange", "Yellow", "Green", "Blue"]
    assert config.data.datasets[0].label == "My First Dataset"
    assert config.data.datasets[0].data == [65, 59, 80, 81, 56]
    assert config.data.datasets[0].fill is False
    assert config.data.datasets[0].backgroundColor == [
        "Red",
        "Orange",
        "Yellow",
        "Green",
        "Blue",
    ]



import pytest

from pydacharts.chart_utils import (
    BarChartOptions,
    DoughnutChartOptions,
    HorizontalBarChartStackedOptions,
    PyramidChartOptions,
)


@pytest.mark.parametrize(
    "element",
    [
        HorizontalBarChartStackedOptions,
        DoughnutChartOptions,
        BarChartOptions,
        PyramidChartOptions,
    ],
)
def test_chart_utils(element):
    element().model_dump_json(exclude_none=True, indent=1)



from enum import Enum
from typing import Literal

from pydantic import BaseModel, Field

from pydacharts.chartjs_types import Color, Font, Function, Padding, number

"""
https://chartjs-plugin-datalabels.netlify.app/guide/options.html

The plugin options can be changed at 3 different levels and are evaluated with the following priority:

per dataset: dataset.datalabels.*
per chart: options.plugins.datalabels.*
or globally: Chart.defaults.plugins.datalabels.*
"""


class Label(BaseModel):
    """
    Represents a DataLabels Label instance
    """

    class Anchor(str, Enum):
        center_ = "center"
        start = "start"
        end = "end"

    class Align(str, Enum):
        center_ = "center"
        start = "start"
        end = "end"
        right = "right"
        bottom = "bottom"
        left = "left"
        top = "top"

    align: Align | None = None
    anchor: Anchor | None = None
    clamp: bool | None = None
    color: Color | None = None
    font: Font | None = None
    formatter: str | None = None
    offset: int | None = None
    opacity: float | None = None

    padding: int | None = None
    borderRadius: int | None = None
    backgroundColor: Color | None = None


"""
https://chartjs-plugin-datalabels.netlify.app/guide/options.html#style-options
Style options are usually inputs for fillStyle or strokeStyle

The following values are supported:
string parsed as CSS color
CanvasGradient
CanvasPattern
"""
DataLabelsPluginStyle = str

align_values = Literal[
    "center",  # (default): the label is centered on the anchor point
    "start",  #: the label is positioned before the anchor point, following the same direction
    "end",  #: the label is positioned after the anchor point, following the same direction
    "right",  #: the label is positioned to the right of the anchor point (0°)
    "bottom",  #: the label is positioned to the bottom of the anchor point (90°)
    "left",  #: the label is positioned to the left of the anchor point (180°)
    "top",  #: the label is positioned to the top of the anchor point (270°)
]

degrees = int


class DataLabelsPlugin(BaseModel):
    align: align_values | degrees | None = None
    anchor: str | None = "center"
    backgroundColor: DataLabelsPluginStyle | None = None
    borderColor: DataLabelsPluginStyle | None = None
    borderRadius: number | None = None
    borderWidth: number | None = None
    clamp: bool | None = None
    clip: bool | None = None
    color: DataLabelsPluginStyle | None = None
    display: Function | Literal[True, False, "auto"] | None = Field(
        True, description="controls the visibility of labels"
    )
    font: Font | None = None
    formatter: Function | None = None
    labels: object | None = None
    listeners: object | None = None
    offset: number | None = Field(
        4,
        description="The offset represents the distance (in pixels) to pull the label away from the anchor point. This option is not applicable when align is 'center'. Also note that if align is 'start', the label is moved in the opposite direction. The default value is 4.",
    )
    opacity: number | None = None
    padding: Padding | None = None
    rotation: degrees | None = None
    textAlign: str | None = None
    textStrokeColor: DataLabelsPluginStyle | None = None
    textStrokeWidth: number | None = None
    textShadowBlur: number | None = None
    textShadowColor: Color | None = None


if __name__ == "__main__":
    print(DataLabelsPlugin().model_dump_json(exclude_none=True))



from pydacharts.models import (
    CartesianTicks,
    ChartType,
    Config,
    Data,
    Dataset,
    Font,
    Options,
    Plugins,
    ScaleOptions,
    Scales,
)
from pydacharts.plugins.datalabels import DataLabelsPlugin, Label


class CenteredDataLabel(DataLabelsPlugin):
    offset: int = 25
    font: Font = Font(size=16, weight="bold")


class OffsetTicks(CartesianTicks):
    mirror: bool = True
    z: int = 100
    padding: int = 10


class RightAlignedScale(ScaleOptions):
    position: str = "right"
    ticks: OffsetTicks = OffsetTicks(mirror=True, display=True, z=100, padding=-10)


class PluginsDataLabels(Plugins):
    datalabels: DataLabelsPlugin = DataLabelsPlugin()


def test_datalabels():
    _data = [
        ("World Bank", 3703888321),
        ("European Union", 1596973970),
        ("UK - Foreign, Commonwealth and Development Office", 1373918906),
        ("Asian Development Bank", 1356078649),
        ("U.S. Agency for International Development", 1101991438),
        ("Government of Germany", 981575439),
        ("World Food Programme", 769002439),
        ("Government of France", 613321247),
        ("Korea EximBank", 598375000),
    ]

    options = Options(
        indexAxis="y",
        scales=Scales(
            y=RightAlignedScale(
                labels=[f"${n[1] / 1e6:.0f} M" for n in _data],
            ),
        ),
        plugins=PluginsDataLabels(
            datalabels=CenteredDataLabel(
                labels=dict(
                    name=Label(
                        anchor=Label.Anchor.start, align=Label.Align.end, clamp=True
                    ),
                )
            )
        ),
    )

    ds = Dataset(
        label="Total Commitment",
        data=[n[1] / 1e6 for n in _data],
    )

    data = Data(labels=[n[0] for n in _data], datasets=[ds])

    config = Config(type=ChartType.bar, options=options, data=data)

    # Assertions
    assert config.type == ChartType.bar
    assert config.data.labels == [n[0] for n in _data]
    assert config.data.datasets[0].label == "Total Commitment"
    assert config.data.datasets[0].data == [n[1] / 1e6 for n in _data]
    assert config.options.indexAxis == "y"
    assert config.options.scales.y.position == "right"
    assert config.options.scales.y.ticks.mirror is True
    assert config.options.scales.y.ticks.z == 100
    assert config.options.scales.y.ticks.padding == -10
    assert config.options.plugins.datalabels.offset == 25
    assert config.options.plugins.datalabels.font.size == 16
    assert config.options.plugins.datalabels.font.weight == "bold"
    assert config.options.plugins.datalabels.labels["name"].anchor == Label.Anchor.start
    assert config.options.plugins.datalabels.labels["name"].align == Label.Align.end
    assert config.options.plugins.datalabels.labels["name"].clamp is True



import calendar
from random import randint

from pydacharts.color import ChartColor, ChartColorTransparent
from pydacharts.models import (
    ChartType,
    Config,
    Data,
    Dataset,
    Options,
    Plugins,
    ScaleOptions,
    Scales,
    Title,
)


def test_center_positioning():
    labels = [calendar.month_name[i] for i in range(1, 8)]
    dataset1_data = [randint(-100, 100) for i in range(1, 8)]
    dataset2_data = [randint(-100, 100) for i in range(1, 8)]

    data = Data(
        labels=labels,
        datasets=[
            Dataset(
                label="Dataset 1",
                data=dataset1_data,
                borderColor=ChartColor.RED,
                backgroundColor=ChartColorTransparent.RED,
            ),
            Dataset(
                label="Dataset 2",
                data=dataset2_data,
                borderColor=ChartColor.BLUE,
                backgroundColor=ChartColorTransparent.BLUE,
            ),
        ],
    )

    options = Options(
        plugins=Plugins(title=Title(display=True, text="Axis Center Positioning")),
        scales=Scales(
            x=ScaleOptions(min=-100, max=100), y=ScaleOptions(min=-100, max=100)
        ),
    )

    config = Config(type=ChartType.scatter, data=data, options=options)

    # Assertions
    assert config.type == ChartType.scatter
    assert config.data.labels == labels
    assert config.data.datasets[0].label == "Dataset 1"
    assert config.data.datasets[0].data == dataset1_data
    assert config.data.datasets[0].borderColor == ChartColor.RED
    assert config.data.datasets[0].backgroundColor == ChartColorTransparent.RED
    assert config.data.datasets[1].label == "Dataset 2"
    assert config.data.datasets[1].data == dataset2_data
    assert config.data.datasets[1].borderColor == ChartColor.BLUE
    assert config.data.datasets[1].backgroundColor == ChartColorTransparent.BLUE
    assert config.options.plugins.title.display is True
    assert config.options.plugins.title.text == "Axis Center Positioning"
    assert config.options.scales.x.min == -100
    assert config.options.scales.x.max == 100
    assert config.options.scales.y.min == -100
    assert config.options.scales.y.max == 100



from pydantic import BaseModel

from pydacharts.models import Font, RgbStr


class Label(BaseModel):
    text: str
    font: Font | None
    color: RgbStr | None


class DoughnutLabel(BaseModel):
    labels: list[Label]



from collections.abc import Sequence
from enum import Enum
from typing import Any, Literal

from pydantic import BaseModel, Field

from pydacharts.chartjs_types import (
    Color,
    Font,
    Function,
    Padding,
    PointStyle,
    RgbStr,
    number,
)
from pydacharts.elements import Elements
from pydacharts.plugins.datalabels import DataLabelsPlugin


class ChartType(str, Enum):
    line = "line"
    bar = "bar"
    horizontalbar = "horizontalBar"
    radar = "radar"
    doughnut = "doughnut"
    pie = "pie"
    polararea = "polarArea"
    bubble = "bubble"
    scatter = "scatter"
    # This is a chart type provided by a plugin
    sankey = "sankey"


class StepOption(str, Enum):
    before = "before"
    after = "after"
    middle = "middle"


class LegendAlign(str, Enum):
    top = "top"
    left = "left"
    bottom = "bottom"
    right = "right"
    chartArea = "chartArea"


class LegendPosition(str, Enum):
    top = "top"
    left = "left"
    bottom = "bottom"
    right = "right"
    chartArea = "chartArea"


class Layout(BaseModel):
    """
    the global options for the chart layout is defined in Chart.defaults.layout
    """

    padding: Padding | None = None


class Dataset(BaseModel):
    label: str | None = None
    data: list[Any] | None = None
    borderColor: list[RgbStr] | RgbStr | None = None
    backgroundColor: list[RgbStr] | RgbStr | None = None
    showLine: bool | None = None
    hoverOffset: int | None = None
    # Support stacked bar charts
    stack: str | None = None
    # Support rounded bars
    borderWidth: int | None = None
    borderRadius: int | None = None
    borderSkipped: bool | None = None
    borderDash: list[int] | None = None
    # Support "combo chart"
    type: ChartType | str | None = None
    order: int | None = None

    datalabels: DataLabelsPlugin | None = None


class LineDataSet(Dataset):
    showLine: bool | None = True
    fill: bool | None = False
    tension: float | None = 0.1
    stepped: bool | StepOption | None = None


class PieDataSet(Dataset):
    fill: bool | None = False


class Data(BaseModel):
    labels: list[str]
    datasets: Sequence[Dataset]


class LineData(Data):
    datasets: Sequence[LineDataSet]


class PieData(Data):
    datasets: Sequence[PieDataSet]


class Title(BaseModel):
    display: bool = True
    text: str | None = None
    color: RgbStr | None = None
    font: Font | None = None
    padding: Padding | None = None
    align: Literal["left", "right", "center"] | None = None


class LegendLabels(BaseModel):
    """
    Namespace: options.plugins.legend.labels
    """

    boxWidth: int | None = Field(40, description="Width of coloured box.")
    boxHeight: int | None = Field(40, description="Height of the coloured box.")
    color: Color | None = Field(
        "black", description="Color of label and the strikethrough."
    )
    font: Font | None = None
    padding: int | None = Field(5, description="Padding between rows of colored boxes.")
    generateLabels: Function | None = Field(
        None,
        description="Generates legend items for each thing in the legend. Default implementation returns the text + styling for the color box. See Legend Item for details.",
    )
    filter: Function | None = Field(
        None,
        description="Filters legend items out of the legend. Receives 2 parameters, a Legend Item and the chart data.",
    )
    sort: Function | None = Field(
        None,
        description="Sorts legend items. Type is : sort(a: LegendItem, b: LegendItem, data: ChartData): number;. Receives 3 parameters, two Legend Items and the chart data. The return value of the function is a number that indicates the order of the two legend item parameters. The ordering matches the return value of Array.prototype.sort()",
    )
    pointStyle: PointStyle | None = Field(
        default=PointStyle.circle,
        description="If specified, this style of point is used for the legend. Only used if usePointStyle is true.",
    )
    textAlign: Literal["left", "right", "center"] | None = Field(
        default="center",
        description="Horizontal alignment of the label text. Options are: 'left', 'right' or 'center'.",
    )
    usePointStyle: bool | None = Field(
        default=False,
        description="Label style will match corresponding point style (size is based on the minimum value between boxWidth and font.size).",
    )


class LegendTitle(BaseModel):
    color: Color | None = Field(None, description="Color of text.")
    display: bool | None = Field(None, description="Is the legend title displayed.")
    font: Font | None = Field(None, description="See Fonts")
    padding: Padding | None = Field(None, description="Padding around the title.")
    text: str | None = Field(None, description="The string title.")


class Legend(BaseModel):
    display: bool | None = Field(None, description="Is the legend shown?")
    position: str | None = Field(None, description="Position of the legend")
    align: str | None = Field(None, description="Alignment of the legend")
    maxHeight: int | None = Field(
        None, description="Maximum height of the legend, in pixels"
    )
    maxWidth: int | None = Field(
        None, description="Maximum width of the legend, in pixels"
    )
    fullSize: bool | None = Field(
        None,
        description="Marks that this box should take the full width/height of the canvas (moving other boxes). This is unlikely to need to be changed in day-to-day use.",
    )
    onClick: Function | None = Field(
        None,
        description="A callback that is called when a click event is registered on a label item. Arguments: [event, legendItem, legend].",
    )
    onHover: Function | None = Field(
        None,
        description="A callback that is called when a 'mousemove' event is registered on top of a label item. Arguments: [event, legendItem, legend].",
    )
    onLeave: Function | None = Field(
        None,
        description="A callback that is called when a 'mousemove' event is registered outside of a previously hovered label item. Arguments: [event, legendItem, legend].",
    )
    reverse: bool | None = Field(
        None, description="Legend will show datasets in reverse order."
    )
    labels: LegendLabels | None = Field(
        None, description="See the Legend Label Configuration section below."
    )
    rtl: bool | None = Field(
        None, description="true for rendering the legends from right to left."
    )
    textDirection: str | None = Field(
        None,
        description="This will force the text direction 'rtl' or 'ltr' on the canvas for rendering the legend, regardless of the css specified on the canvas",
    )
    title: LegendTitle | None = Field(
        None, description="See the Legend Title Configuration section below."
    )


class TooltipCallbacks(BaseModel):
    beforeTitle: Function | None = Field(
        None, description="Returns the text to render before the title."
    )
    title: Function | None = Field(
        None, description="Returns text to render as the title of the tooltip."
    )
    afterTitle: Function | None = Field(
        None, description="Returns text to render after the title."
    )
    beforeBody: Function | None = Field(
        None, description="Returns text to render before the body section."
    )
    beforeLabel: Function | None = Field(
        None,
        description="Returns text to render before an individual label. This will be called for each item in the tooltip.",
    )
    label: Function | None = Field(
        None,
        description="Returns text to render for an individual item in the tooltip. more...",
    )
    labelColor: Function | None = Field(
        None, description="Returns the colors to render for the tooltip item. more..."
    )
    labelTextColor: Function | None = Field(
        None,
        description="Returns the colors for the text of the label for the tooltip item.",
    )
    labelPointStyle: Function | None = Field(
        None,
        description="Returns the point style to use instead of color boxes if usePointStyle is true (object with values pointStyle and rotation). Default implementation uses the point style from the dataset points. more...",
    )
    afterLabel: Function | None = Field(
        None, description="Returns text to render after an individual label."
    )
    afterBody: Function | None = Field(
        None, description="Returns text to render after the body section."
    )
    beforeFooter: Function | None = Field(
        None, description="Returns text to render before the footer section."
    )
    footer: Function | None = Field(
        None, description="Returns text to render as the footer of the tooltip."
    )
    afterFooter: Function | None = Field(
        None, description="Text to render after the footer section."
    )


class Tooltip(BaseModel):
    """
    https://www.chartjs.org/docs/latest/configuration/tooltip.html#tooltip-callbacks
    """

    enabled: bool | None = Field(None, description="Are on-canvas tooltips enabled?")
    external: Function | None = Field(None, description="See external tooltip section.")
    mode: str | None = Field(
        None, description="Sets which elements appear in the tooltip."
    )
    intersect: bool | None = Field(
        None,
        description="If true, the tooltip mode applies only when the mouse position intersects with an element. If false, the mode will be applied at all times.",
    )
    position: str | None = Field(
        None, description="The mode for positioning the tooltip."
    )
    callbacks: TooltipCallbacks | None = Field(
        None, description="See the callbacks section."
    )
    itemSort: Function | None = Field(None, description="Sort tooltip items.")
    filter: Function | None = Field(None, description="Filter tooltip items.")
    backgroundColor: Color | None = Field(
        None, description="Background color of the tooltip."
    )
    titleColor: Color | None = Field(None, description="Color of title text.")
    titleFont: Font | None = Field(None, description="See Fonts.")
    titleAlign: str | None = Field(
        None, description="Horizontal alignment of the title text lines."
    )
    titleSpacing: number | None = Field(
        None, description="Spacing to add to top and bottom of each title line."
    )
    titleMarginBottom: number | None = Field(
        None, description="Margin to add on bottom of title section."
    )
    bodyColor: Color | None = Field(None, description="Color of body text.")
    bodyFont: Font | None = Field(None, description="See Fonts.")
    bodyAlign: str | None = Field(
        None, description="Horizontal alignment of the body text lines."
    )
    bodySpacing: int | None = Field(
        None, description="Spacing to add to top and bottom of each tooltip item."
    )
    footerColor: Color | None = Field(None, description="Color of footer text.")
    footerFont: Font | None = Field(None, description="See Fonts.")
    footerAlign: str | None = Field(
        None, description="Horizontal alignment of the footer text lines."
    )
    footerSpacing: int | None = Field(
        None, description="Spacing to add to top and bottom of each footer line."
    )
    footerMarginTop: int | None = Field(
        None, description="Margin to add before drawing the footer."
    )
    padding: Padding | None = Field(None, description="Padding inside the tooltip.")
    caretPadding: int | None = Field(
        None,
        description="Extra distance to move the end of the tooltip arrow away from the tooltip point.",
    )
    caretSize: int | None = Field(
        None, description="Size, in px, of the tooltip arrow."
    )
    cornerRadius: int | None = Field(
        None, description="Radius of tooltip corner curves."
    )
    multiKeyBackground: Color | None = Field(
        None,
        description="Color to draw behind the colored boxes when multiple items are in the tooltip.",
    )
    displayColors: bool | None = Field(
        None, description="If true, color boxes are shown in the tooltip."
    )
    boxWidth: int | None = Field(
        None, description="Width of the color box if displayColors is true."
    )
    boxHeight: int | None = Field(
        None, description="Height of the color box if displayColors is true."
    )
    boxPadding: int | None = Field(
        None, description="Padding between the color box and the text."
    )
    usePointStyle: int | None = Field(
        None,
        description="Use the corresponding point style (from dataset options) instead of color boxes, ex: star, triangle etc. (size is based on the minimum value between boxWidth and boxHeight).",
    )
    borderColor: Color | None = Field(None, description="Color of the border.")
    borderWidth: int | None = Field(None, description="Size of the border.")
    rtl: bool | None = Field(
        None, description="true for rendering the tooltip from right to left."
    )
    textDirection: Literal["rtl", "ltr"] | None = Field(
        None,
        description="This will force the text direction 'rtl' or 'ltr on the canvas for rendering the tooltips, regardless of the css specified on the canvas",
    )
    xAlign: str | None = Field(
        None, description="Position of the tooltip caret in the X direction."
    )
    yAlign: str | None = Field(
        None, description="Position of the tooltip caret in the Y direction."
    )


class Plugins(BaseModel):
    legend: Legend | None = None
    title: Title | None = None
    datalabels: DataLabelsPlugin | list[DataLabelsPlugin] | None = None
    tooltip: Tooltip | None = None


class Ticks(BaseModel):
    """
    Namespace: options.scales[scaleId].ticks
    """

    beginAtZero: bool | None = None
    backdropColor: Color | None = Field(
        default=None, description="Color of label backdrops."
    )
    backdropPadding: Padding | None = Field(
        default=None, description="Padding of label backdrop."
    )
    callback: Function | None = Field(
        default=None,
        description="Returns the string representation of the tick value as it should be displayed on the chart..",
    )
    display: bool | None = Field(default=None, description="If true, show tick labels.")
    color: Color | None = Field(default=None, description="Color of ticks.")
    font: Font | None = None
    major: dict | None = Field(default=None, description="Major ticks configuration.")
    padding: int | None = Field(
        default=None, description="Sets the offset of the tick labels from the axis"
    )
    showLabelBackdrop: bool | None = Field(
        default=None, description="If true, draw a background behind the tick labels."
    )
    stepSize: int | None = None
    textStrokeColor: Color | None = Field(
        default=None, description="The color of the stroke around the text."
    )
    textStrokeWidth: int | None = Field(
        default=None, description="Stroke width around the text."
    )
    z: int | None = Field(
        default=None,
        description="z-index of tick layer. Useful when ticks are drawn on chart area. Values <= 0 are drawn under datasets, > 0 on top.",
    )


class Grid(BaseModel):
    borderColor: Color | None = Field(
        default=None, description="The color of the border line."
    )
    borderWidth: float | None = Field(
        default=None, description="The width of the border line."
    )
    borderDash: list[float] | None = Field(
        default=None, description="Length and spacing of dashes on grid lines. See MDN."
    )
    borderDashOffset: float | None = Field(
        default=None, description="Offset for line dashes. See MDN."
    )
    circular: bool | None = Field(
        default=None,
        description="If true, gridlines are circular (on radar chart only).",
    )
    color: Color | None = Field(
        default=None,
        description="The color of the grid lines. If specified as an array, the first color applies to the first grid line, the second to the second grid line, and so on.",
    )
    display: bool | None = Field(
        default=None, description="If false, do not display grid lines for this axis."
    )
    drawBorder: bool | None = Field(
        default=None,
        description="If true, draw a border at the edge between the axis and the chart area.",
    )
    drawOnChartArea: bool | None = Field(
        default=None,
        description="If true, draw lines on the chart area inside the axis lines. This is useful when there are multiple axes and you need to control which grid lines are drawn.",
    )
    drawTicks: bool | None = Field(
        default=None,
        description="If true, draw lines beside the ticks in the axis area beside the chart.",
    )
    lineWidth: float | None = Field(
        default=None, description="Stroke width of grid lines."
    )
    offset: bool | None = Field(
        default=None,
        description="If true, grid lines will be shifted to be between labels. This is set to true for a bar chart by default.",
    )
    tickBorderDash: list[float] | None = Field(
        default=None,
        description="Length and spacing of the tick mark line. If not set, defaults to the grid line borderDash value.",
    )
    tickBorderDashOffset: float | None = Field(
        default=None,
        description="Offset for the line dash of the tick mark. If unset, defaults to the grid line borderDashOffset value",
    )
    tickColor: Color | None = Field(
        default=None,
        description="Color of the tick line. If unset, defaults to the grid line color.",
    )
    tickLength: int | None = Field(
        default=None,
        description="Length in pixels that the grid lines will draw into the axis area.",
    )
    tickWidth: int | None = Field(
        default=None,
        description="Width of the tick mark in pixels. If unset, defaults to the grid line width.",
    )
    z: int | None = Field(
        default=None,
        description="z-index of gridline layer. Values <= 0 are drawn under datasets, > 0 on top.",
    )


class CartesianTicks(Ticks):
    """
    Namespace: options.scales[scaleId].ticks
    """

    align: str | None = Field(
        default=None,
        description="The tick alignment along the axis. Can be 'start', 'center', or 'end'.",
    )
    crossAlign: str | None = Field(
        default=None,
        description="The tick alignment perpendicular to the axis. Can be 'near', 'center', or 'far'. See Tick Alignment",
    )
    sampleSize: int | None = Field(
        default=None,
        description="The number of ticks to examine when deciding how many labels will fit. Setting a smaller value will be faster, but may be less accurate when there is large variability in label length.",
    )
    autoSkip: bool | None = Field(
        default=None,
        description="If true, automatically calculates how many labels can be shown and hides labels accordingly. Labels will be rotated up to maxRotation before skipping any. Turn autoSkip off to show all labels no matter what.",
    )
    autoSkipPadding: int | None = Field(
        default=None,
        description="Padding between the ticks on the horizontal axis when autoSkip is enabled.",
    )
    includeBounds: bool | None = Field(
        default=None,
        description='Should the defined min and max values be presented as ticks even if they are not "nice".',
    )
    labelOffset: int | None = Field(
        default=None,
        description="Distance in pixels to offset the label from the centre point of the tick (in the x-direction for the horizontal axis, and the y-direction for the vertical axis).",
    )
    minRotation: int | None = Field(
        default=None, description="Minimum rotation for tick labels, in degrees."
    )
    maxRotation: int | None = Field(
        default=None, description="Maximum rotation for tick labels, in degrees."
    )
    mirror: bool | None = Field(
        default=None,
        description="Flips tick labels around axis, displaying the labels inside the chart instead of outside.",
    )
    padding: int | None = Field(
        default=None, description="Padding between the tick label and the axis."
    )
    showLabelBackdrop: bool | None = Field(
        default=None, description="If true, draw a background behind the tick labels."
    )
    textStrokeColor: Color | None = Field(
        default=None, description="The color of the stroke around the text."
    )
    textStrokeWidth: int | None = Field(
        default=None, description="Stroke width around the text."
    )
    z: int | None = Field(
        default=None,
        description="z-index of tick layer. Useful when ticks are drawn on chart area. Values <= 0 are drawn under datasets, > 0 on top.",
    )


class ScaleOptions(BaseModel):
    display: bool | None = True
    beginAtZero: bool | None = None
    title: Title | None = None
    min: int | None = None
    max: int | None = None
    stacked: bool | None = None
    position: str | None = None
    offset: bool = True
    ticks: Ticks | CartesianTicks | None = None
    barThickness: int | None = None
    labels: Any | None = None
    grid: Grid | None = None
    type: str | None = None


class Scales(BaseModel):
    y: ScaleOptions | None = None
    x: ScaleOptions | None = None
    # Added for datalabels plugin
    xAxes: list[ScaleOptions] | None = None
    yAxes: list[ScaleOptions] | None = None
    # x1, y1 represent additional
    y1: ScaleOptions | None = None
    x1: ScaleOptions | None = None
    grid: Grid | None = None


class Interaction(BaseModel):
    intersect: bool | None = None  # False for stacked charts
    mode: str | None = Field(
        None,
        description="Sets which elements appear in the interaction. See Interaction Modes for details.",
    )
    axis: str | None = "x"
    includeInvisible: bool | None = False


class Options(BaseModel):
    responsive: bool | None = Field(
        True,
        description="Resizes the chart canvas when its container does (important note...).",
    )
    maintainAspectRatio: bool | None = Field(
        True,
        description="Maintain the original canvas aspect ratio (width / height) when resizing.",
    )
    aspectRatio: number | Function | None = Field(
        2,
        description="Canvas aspect ratio (i.e. width / height, a value of 1 representing a square canvas). Note that this option is ignored if the height is explicitly defined either as attribute or via the style.",
    )
    onResize: Function | None = Field(
        None,
        description="Called when a resize occurs. Gets passed two arguments: the chart instance and the new size.",
    )
    resizeDelay: number | None = Field(
        0,
        description="Delay the resize update by give amount of milliseconds. This can ease the resize process by debouncing update of the elements.",
    )

    plugins: Plugins | None = None
    scales: Scales | None = None
    elements: Elements | None = None
    indexAxis: str | None = None
    interaction: Interaction | None = None

    # Bar chart option
    barPercentage: float | None = None
    layout: Layout | None = None
    stacked: bool | None = False

    # Doughnut chart
    cutout: str | None = None


class Config(BaseModel):
    type: ChartType | str = ChartType.line
    data: Data | None = None
    options: Options | None = None


if __name__ == "__main__":
    Layout().model_dump_json()
    Dataset().model_dump_json()
    Data(labels=["1", "2", "3"], datasets=[Dataset()]).model_dump_json()
    Font().model_dump_json()
    Title().model_dump_json()
    LegendLabels().model_dump_json()
    LegendTitle().model_dump_json()
    Legend().model_dump_json()
    DataLabelsPlugin().model_dump_json()
    TooltipCallbacks().model_dump_json()
    Tooltip().model_dump_json()
    Plugins().model_dump_json()
    Ticks().model_dump_json()
    Grid().model_dump_json()
    ScaleOptions().model_dump_json()
    Scales().model_dump_json()
    Interaction().model_dump_json()
    Options().model_dump_json()
    Config().model_dump_json()



from pydacharts.models import (
    ChartType,
    Config,
    Data,
    Dataset,
    Options,
    ScaleOptions,
    Scales,
)
from pydacharts.plugins.datalabels import Label
from tests.test_datalabels import CenteredDataLabel, OffsetTicks, PluginsDataLabels


class RightAlignedScale(ScaleOptions):
    position: str = "right"
    ticks: OffsetTicks = OffsetTicks(mirror=True, display=True, z=100, padding=-10)


def config():
    _data = [
        ("World Bank", 3703888321),
        ("European Union", 1596973970),
        ("UK - Foreign, Commonwealth and Development Office", 1373918906),
        ("Asian Development Bank", 1356078649),
        ("U.S. Agency for International Development", 1101991438),
        ("Government of Germany", 981575439),
        ("World Food Programme", 769002439),
        ("Government of France", 613321247),
        ("Korea EximBank", 598375000),
    ]

    options = Options(
        indexAxis="y",
        scales=Scales(
            y=RightAlignedScale(
                labels=[f"${n[1] / 1e6:.0f} M" for n in _data],
            ),
        ),
        plugins=PluginsDataLabels(
            datalabels=CenteredDataLabel(
                labels=dict(
                    name=Label(
                        anchor=Label.Anchor.start, align=Label.Align.end, clamp=True
                    ),
                )
            )
        ),
    )

    ds = Dataset(
        label="Total Commitment",
        data=[n[1] / 1e6 for n in _data],
    )

    data = Data(labels=[n[0] for n in _data], datasets=[ds])

    return Config(type=ChartType.bar, options=options, data=data)



import calendar
from random import randint

from pydacharts.color import ChartColor, ChartColorTransparent
from pydacharts.models import (
    ChartType,
    Config,
    Data,
    Dataset,
    Interaction,
    Legend,
    Options,
    Plugins,
    ScaleOptions,
    Scales,
    Title,
)


def test_vertical_bar():
    bar_data = Data(
        labels=[calendar.month_name[i] for i in range(1, 8)],
        datasets=[
            Dataset(
                label="Dataset 1",
                data=[randint(-100, 100) for i in range(1, 8)],
                borderColor=ChartColor.RED,
                backgroundColor=ChartColorTransparent.RED,
            ),
            Dataset(
                label="Dataset 2",
                data=[randint(-100, 100) for i in range(1, 8)],
                borderColor=ChartColor.BLUE,
                backgroundColor=ChartColorTransparent.BLUE,
            ),
        ],
    )
    options = Options(
        plugins=Plugins(
            title=Title(text="Chart.js Bar Chart"), legend=Legend(position="top")
        ),
        scales=Scales(y=ScaleOptions(beginAtZero=True)),
    )

    config = Config(type=ChartType.bar, data=bar_data, options=options)
    assert config.type == ChartType.bar
    assert config.data == bar_data
    assert config.options == options


def test_horizontal_bar():
    data = Data(
        labels=[calendar.month_name[i] for i in range(1, 8)],
        datasets=[
            Dataset(
                label="Dataset 1",
                data=[randint(-100, 100) for i in range(1, 8)],
                borderColor=ChartColor.RED,
                backgroundColor=ChartColorTransparent.RED,
            ),
            Dataset(
                label="Dataset 2",
                data=[randint(-100, 100) for i in range(1, 8)],
                borderColor=ChartColor.BLUE,
                backgroundColor=ChartColorTransparent.BLUE,
            ),
        ],
    )
    options = Options(
        indexAxis="y",
        elements=dict(bar=dict(borderWidth=2)),
        plugins=Plugins(
            legend=Legend(position="right"),
            title=Title(text="Chart.js Horizontal Bar Chart"),
        ),
        scales=Scales(x=ScaleOptions(stacked=True), y=ScaleOptions(stacked=True)),
    )

    config = Config(type=ChartType.bar, data=data, options=options)
    assert config.type == ChartType.bar
    assert config.data == data
    assert config.options == options


def test_stacked_bar():
    data = Data(
        labels=[calendar.month_name[i] for i in range(1, 8)],
        datasets=[
            Dataset(
                label="Dataset 1",
                data=[randint(-100, 100) for i in range(1, 8)],
                borderColor=ChartColor.RED,
                backgroundColor=ChartColor.RED,
                stack="Stack 0",
            ),
            Dataset(
                label="Dataset 2",
                data=[randint(-100, 100) for i in range(1, 8)],
                borderColor=ChartColor.BLUE,
                backgroundColor=ChartColor.BLUE,
                stack="Stack 0",
            ),
            Dataset(
                label="Dataset 3",
                data=[randint(-100, 100) for i in range(1, 8)],
                borderColor=ChartColor.GREEN,
                backgroundColor=ChartColor.GREEN,
                stack="Stack 1",
            ),
        ],
    )

    options = Options(
        plugins=Plugins(
            title=Title(display=True, text="Charts.js Bar Chart - Grouped")
        ),
        interaction=Interaction(intersect=False),
        scales=Scales(x=ScaleOptions(stacked=True), y=ScaleOptions(stacked=True)),
    )

    config = Config(type=ChartType.bar, options=options, data=data)
    assert config.type == ChartType.bar
    assert config.data == data
    assert config.options == options


def test_floating_bar():
    data = Data(
        labels=[calendar.month_name[i] for i in range(1, 8)],
        datasets=[
            Dataset(
                label="Dataset 1",
                data=[[randint(-100, 100), randint(-100, 100)] for i in range(1, 8)],
                backgroundColor=ChartColor.RED,
            ),
            Dataset(
                label="Dataset 2",
                data=[[randint(-100, 100), randint(-100, 100)] for i in range(1, 8)],
                backgroundColor=ChartColor.BLUE,
            ),
        ],
    )

    options = Options(
        plugins=Plugins(
            legend=Legend(position="top"),
            title=Title(display=True, text="Chart.js Floating Bar Chart"),
        ),
    )

    config = Config(type=ChartType.bar, options=options, data=data)
    assert config.type == ChartType.bar
    assert config.data == data
    assert config.options == options


def test_rounded_bar():
    data = Data(
        labels=[calendar.month_name[i] for i in range(1, 8)],
        datasets=[
            Dataset(
                label="Fully Rounded",
                data=[randint(-100, 100) for i in range(1, 8)],
                borderColor=ChartColor.BLUE,
                backgroundColor=ChartColorTransparent.BLUE,
                borderWidth=2,
                borderRadius=100,
                borderSkipped=False,
            ),
            Dataset(
                label="Small Radius",
                data=[randint(-100, 100) for i in range(1, 8)],
                borderColor=ChartColor.RED,
                backgroundColor=ChartColorTransparent.RED,
                borderWidth=2,
                borderRadius=5,
                borderSkipped=False,
            ),
        ],
    )

    options = Options(
        plugins=Plugins(
            legend=Legend(position="top"),
            title=Title(display=True, text="Chart.js Bar Chart"),
        ),
    )

    config = Config(type=ChartType.bar, options=options, data=data)
    assert config.type == ChartType.bar
    assert config.data == data
    assert config.options == options



import pytest

from pydacharts.models import (
    Config,
    Data,
    Dataset,
    Font,
    Grid,
    Interaction,
    Layout,
    Legend,
    LegendLabels,
    LegendTitle,
    Options,
    Plugins,
    ScaleOptions,
    Scales,
    Ticks,
    Title,
    Tooltip,
    TooltipCallbacks,
)
from pydacharts.plugins.datalabels import DataLabelsPlugin


@pytest.mark.parametrize(
    "testcase",
    [
        Layout,
        Font,
        Title,
        LegendLabels,
        LegendTitle,
        Legend,
        DataLabelsPlugin,
        TooltipCallbacks,
        Tooltip,
        Plugins,
        Ticks,
        Grid,
        ScaleOptions,
        Scales,
        Interaction,
        Options,
        Config,
    ],
)
def test_all_models(testcase):
    testcase().model_dump_json()


def test_data_labels():
    Data(labels=["1", "2", "3"], datasets=[Dataset()]).model_dump_json()
